var _httpTimeout;

var conf = {
	serverPrefix: 'http://localhost/tloms/',
	mainEndpointUrl: 'MyTime',
	log: true
};

String.prototype.hashCode = function() {
  var hash = 0, i, chr, len;
  if (this.length == 0) return hash;
  for (i = 0, len = this.length; i < len; i++) {
    chr   = this.charCodeAt(i);
    hash  = ((hash << 5) - hash) + chr;
    hash |= 0; // Convert to 32bit integer
  }
  return hash;
};

function addParameterToURL(str_url, param, value) {
 	param = param + '=' + value;
	str_url += (str_url.split('?')[1] ? '&':'?') + param;
	return str_url;
}

function loadRemote(str_class, str_url, successCallback, errorCallback, metadata, deviceType) {
	syncWorker.logger.log('loadRemote, class: ' + str_class + ', url: ' + str_url);
	var xmlHttpTimeout;
	var xhr;
	var headers = {};

	if (typeof XMLHttpRequest !== 'undefined') xhr = new XMLHttpRequest();
	else {
		var versions = ["MSXML2.XmlHttp.5.0",
			 	"MSXML2.XmlHttp.4.0",
			 	"MSXML2.XmlHttp.3.0",
			 	"MSXML2.XmlHttp.2.0",
			 	"Microsoft.XmlHttp"]

		for(var i = 0, len = versions.length; i < len; i++) {
		try {
			xhr = new ActiveXObject(versions[i]);
			break;
		}
			catch(e){}
		} // end for
	}

	var timeoutControl = new Date();
	xhr.onreadystatechange = ensureReadiness;
	xhr.withCredentials = true;

	function ensureReadiness() {
		if(xhr.readyState < 4) {
			return;
		}
		xhr.responseTime = ((new Date()) - timeoutControl);

		// all is well
		if(xhr.readyState === 4) {
			clearTimeout(xmlHttpTimeout);
			if(xhr.status >= 200 && xhr.status < 400) {
				successCallback(str_class, xhr);
			} else {
				errorCallback(str_class, xhr);
			}

		}
	}
	xhr.failedURL = str_url;
	//Add a timestamp to the url (cache avoid)
	var tstamp = new Date().getTime();
	var param = 'd';
	str_url = addParameterToURL(str_url, param, tstamp);

	xhr.open('GET', str_url, true);

	// Adds the 'Accept' field to the HTTP header for content negotiation
	xhr.setRequestHeader('Accept', formatContentNegotiation(metadata));
	// Adds the 'X-Platform' field to the HTTP header for device type
	xhr.setRequestHeader('X-PLATFORM', deviceType);

	xhr.send('');

	xmlHttpTimeout = setTimeout(ajaxTimeout, _httpTimeout);
	function ajaxTimeout() {
		xhr.abort();
	}
}

/**
 * Creates the content negotiation field value based on the data in metadata.
 *
 * @param  {String} metadata
 * @return {String}
 */
function formatContentNegotiation(metadata) {
    var ver, type, mime;

    ver = metadata.apiVer || 'v1';
    type = metadata.type || 'application/vnd.paychex.com';
    mime = metadata.mime || 'json';

    return mime + '+' + ver + '+' + type;
}


var syncWorker = {

	logger: {
		log: function(str_location, str_event) {
			if (str_event == undefined) {
				str_event = '';
			} else {
				str_event = ' - ' + str_event;
			}

			if (conf.log) {
				try {
					console.log("WebWorker - " + str_location + str_event);
				} catch(err) {
				}
			}
		}
	},

	setLoggerLevel: function(options) {
		if (options !== null && options !== undefined && options.hasOwnProperty('devMode') && options.devMode) {
			conf.log = true;
		} else {
			conf.log = false;
		}
	},

	communication: {

		sendDataMessage: function(str_event, str_class, data) {
			var message = {
				message: str_event,
				payload: {
					class: str_class,
					data: data
				}
			};

			self.postMessage(message);
		}
	},

	setup: function() {
		self.addEventListener('message', function(e) {
			switch(e.data.message) {
				case 'ENDPOINTS_UPDATE':
					self.postMessage({ message: 'LOG', payload: 'SyncWorker: Received ENDPOINTS_UPDATE request' });
					syncWorker.setLoggerLevel(e.data.payload);
					syncWorker.endpoints.setup(e.data.payload, e.data.metadata);
				break;

				case 'SNAPSHOT_UPDATE':
					self.postMessage({ message: 'LOG', payload: 'SyncWorker: Received SNAPSHOT_UPDATE request' });
					syncWorker.setLoggerLevel(e.data.payload);
					syncWorker.endpoints.snapshotUpdate(e.data.payload, e.data.metadata);
				break;

				case 'REQUEST_CANCEL':
					self.postMessage({ message: 'LOG', payload: 'SyncWorker: Received REQUEST_CANCEL request' });
					try {
						self.close();
					} catch(err) {

					}
				break;

				case 'SET_TIMEOUT':
					_httpTimeout = e.data.timeout;
				break;
			}

		}, false);
	},

	endpoints: {
		setup: function(data, metadata) {
			var str_class = data.class;
			var str_url = data.url;

			syncWorker.logger.log('SyncWorker->endpoints->setup');

			syncWorker.logger.log("SETTING UP ENDPOINT URL: " + str_url);
			syncWorker.endpoints.fetch(str_class, str_url, metadata);
		},

		processXhrResponse: function(xhr) {
			var str_result = xhr.responseText;
			var payload = {
				hash: '',
				status: xhr.status,
				responseTime: xhr.responseTime,
				responseURL: xhr.responseURL
			};
			try {
				var parsed_data = JSON.parse(str_result);
				payload.data = parsed_data;
				payload.hash = str_result.hashCode();
			} catch(err) {
				payload.data = null;
			}
			return payload;
		},

		processXhrErrorResponse: function(xhr) {
			var str_result = xhr.responseText;
			var payload = {
				status: xhr.status,
				responseTime: xhr.responseTime,
				responseURL: xhr.responseURL ? xhr.responseURL : xhr.failedURL,
				data: str_result
			};

			return payload;
		},

		fetch: function(str_class, str_url, metadata) {
			syncWorker.logger.log('SyncWorker->endpoints->fetch', "class: " + str_class);

			loadRemote(str_class, str_url, syncWorker.endpoints.fetchCallback, syncWorker.endpoints.fetchErrorCallback, metadata);
		},

		fetchCallback: function(str_class, xhr) {
			syncWorker.logger.log('SyncWorker->endpoints->fetchCallback', "class: " + str_class);

			var payload = syncWorker.endpoints.processXhrResponse(xhr);
			payload.class = 'TloEndpoints';
			if (payload.data === null) {
				syncWorker.communication.sendDataMessage('ENDPOINTS_UPDATE_ERROR', str_class, payload);
			} else {
				syncWorker.communication.sendDataMessage('ENDPOINTS_UPDATE_RESPONSE', str_class, payload);
			}
		},

		fetchErrorCallback: function(str_class, xhr) {
			syncWorker.logger.log('SyncWorker->endpoints->fetchErrorCallback', "class: " + str_class + ", status:" + xhr.status + ", responseText: " + xhr.responseText);

			var payload = syncWorker.endpoints.processXhrErrorResponse(xhr);
			syncWorker.communication.sendDataMessage('ENDPOINTS_UPDATE_ERROR', str_class, payload);
		},

		snapshotUpdate: function(data, metadata) {
			syncWorker.logger.log('SyncWorker->snapshotUpdate', "class: " + data.class);

			var str_class = data.class;
			var str_url = data.url;
			loadRemote(str_class, str_url, syncWorker.endpoints.snapshotUpdateCallback, syncWorker.endpoints.snapshotUpdateErrorCallback, metadata, data.deviceType);
		},

		snapshotUpdateCallback: function(str_class, xhr) {
			syncWorker.logger.log('SyncWorker->endpoints->snapshotUpdateCallback', "class: " + str_class);

			var payload = syncWorker.endpoints.processXhrResponse(xhr);
			if (payload.data === null) {
				syncWorker.communication.sendDataMessage('SNAPSHOT_UPDATE_ERROR', str_class, payload);
			} else {
				syncWorker.communication.sendDataMessage('SNAPSHOT_UPDATE_RESPONSE', str_class, payload);
			}
		},

		snapshotUpdateErrorCallback: function(str_class, xhr) {
			syncWorker.logger.log('SyncWorker->endpoints->snapshotUpdateErrorCallback', "class: " + str_class + ", status:" + xhr.status + ", responseText: " + xhr.responseText);

			var payload = syncWorker.endpoints.processXhrErrorResponse(xhr);
			syncWorker.communication.sendDataMessage('SNAPSHOT_UPDATE_ERROR', str_class, payload);
		}
	}
};


syncWorker.setup();
